@@ -1,13 +1,32 @@
+--[[
+Copyright (c) 2018, Vsevolod Stakhov <vsevolod@highsecure.ru>
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+]]--
+
+if not rspamd_config:has_torch() then
+  return
+end
+
 local torch = require "torch"
 local nn = require "nn"
 local lua_util = require "lua_util"
 local ucl = require "ucl"
 local logger = require "rspamd_logger"
-local getopt = require "rspamadm/getopt"
 local optim = require "optim"
 local rspamd_util = require "rspamd_util"
-
-local rescore_utility = require "rspamadm/rescore_utility"
+local argparse = require "argparse"
+local rescore_utility = require "rescore_utility"
 
 local opts
 local ignore_symbols = {
@@ -15,6 +34,104 @@ local ignore_symbols = {
   ['DATE_IN_FUTURE'] = true,
 }
 
+local parser = argparse()
+    :name "rspamadm rescore"
+    :description "Estimate optimal symbol weights from log files"
+    :help_description_margin(37)
+
+parser:option "-l --log"
+      :description "Log file or files (from rescore)"
+      :argname("<log>")
+      :args "*"
+parser:option "-c --config"
+      :description "Path to config file"
+      :argname("<file>")
+      :default(rspamd_paths["CONFDIR"] .. "/" .. "rspamd.conf")
+parser:option "-o --output"
+      :description "Output file"
+      :argname("<file>")
+      :default("new.scores")
+parser:flag "-d --diff"
+      :description "Show differences in scores"
+parser:flag "-v --verbose"
+      :description "Verbose output"
+parser:flag "-z --freq"
+      :description "Display hit frequencies"
+parser:option "-i --iters"
+      :description "Learn iterations"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(10)
+parser:option "-b --batch"
+      :description "Batch size"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(100)
+parser:option "-d --decay"
+      :description "Decay rate"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.001)
+parser:option "-m --momentum"
+      :description "Learn momentum"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.1)
+parser:option "-t --threads"
+      :description "Number of threads to use"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(1)
+parser:option "-o --optim"
+      :description "Optimisation algorithm"
+      :argname("<alg>")
+      :convert {
+        LBFGS = "LBFGS",
+        ADAM = "ADAM",
+        ADAGRAD = "ADAGRAD",
+        SGD = "SGD",
+        NAG = "NAG"
+      }
+      :default "ADAM"
+parser:option "--ignore-symbol"
+      :description "Ignore symbol from logs"
+      :argname("<sym>")
+      :args "*"
+parser:option "--penalty-weight"
+      :description "Add new penalty weight to test"
+      :argname("<n>")
+      :convert(tonumber)
+      :args "*"
+parser:option "--learning-rate"
+      :description "Add new learning rate to test"
+      :argname("<n>")
+      :convert(tonumber)
+      :args "*"
+parser:option "--spam_action"
+      :description "Spam action"
+      :argname("<act>")
+      :default("reject")
+parser:option "--learning_rate_decay"
+      :description "Learn rate decay (for some algs)"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.0)
+parser:option "--weight_decay"
+      :description "Weight decay (for some algs)"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.0)
+parser:option "--l1"
+      :description "L1 regularization penalty"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.0)
+parser:option "--l2"
+      :description "L2 regularization penalty"
+      :argname("<n>")
+      :convert(tonumber)
+      :default(0.0)
+
 local function make_dataset_from_logs(logs, all_symbols, spam_score)
   -- Returns a list of {input, output} for torch SGD train
 
@@ -71,17 +188,18 @@ local function init_weights(all_symbols, original_symbol_scores)
   return weights
 end
 
-local function shuffle(logs)
+local function shuffle(logs, messages)
 
   local size = #logs
   for i = size, 1, -1 do
     local rand = math.random(size)
     logs[i], logs[rand] = logs[rand], logs[i]
+    messages[i], messages[rand] = messages[rand], messages[i]
   end
 
 end
 
-local function split_logs(logs, split_percent)
+local function split_logs(logs, messages, split_percent)
 
   if not split_percent then
     split_percent = 60
@@ -91,16 +209,20 @@ local function split_logs(logs, split_percent)
 
   local test_logs = {}
   local train_logs = {}
+  local test_messages = {}
+  local train_messages = {}
 
   for i=1,split_index do
-    train_logs[#train_logs + 1] = logs[i]
+    table.insert(train_logs, logs[i])
+    table.insert(train_messages, messages[i])
   end
 
   for i=split_index + 1, #logs do
-    test_logs[#test_logs + 1] = logs[i]
+    table.insert(test_logs, logs[i])
+    table.insert(test_messages, messages[i])
   end
 
-  return train_logs, test_logs
+  return {train_logs,train_messages}, {test_logs,test_messages}
 end
 
 local function stitch_new_scores(all_symbols, new_scores)
@@ -174,7 +296,10 @@ local function print_score_diff(new_symbol_scores, original_symbol_scores)
 
 end
 
-local function calculate_fscore_from_weights(logs, all_symbols, weights, threshold)
+local function calculate_fscore_from_weights(logs, messages,
+                                             all_symbols,
+                                             weights,
+                                             threshold)
 
   local new_symbol_scores = weights:clone()
 
@@ -182,14 +307,16 @@ local function calculate_fscore_from_weights(logs, all_symbols, weights, thresho
 
   logs = update_logs(logs, new_symbol_scores)
 
-  local file_stats, _, all_fps, all_fns = rescore_utility.generate_statistics_from_logs(logs, threshold)
+  local file_stats, _, all_fps, all_fns =
+      rescore_utility.generate_statistics_from_logs(logs, messages, threshold)
 
   return file_stats.fscore, all_fps, all_fns
 end
 
-local function print_stats(logs, threshold)
+local function print_stats(logs, messages, threshold)
 
-  local file_stats, _ = rescore_utility.generate_statistics_from_logs(logs, threshold)
+  local file_stats, _ = rescore_utility.generate_statistics_from_logs(logs,
+      messages, threshold)
 
   local file_stat_format = [[
 F-score: %.2f
@@ -226,7 +353,7 @@ local function train(dataset, opt, model, criterion, epoch,
   local lbfgsState
   local sgdState
 
-  local batch_size = opt.batch_size
+  local batch_size = opt.batch
 
   logger.messagex("trainer epoch #%s, %s batch", epoch, batch_size)
 
@@ -300,7 +427,7 @@ local function train(dataset, opt, model, criterion, epoch,
     end
 
     -- optimize on current mini-batch
-    if opt.optimization == 'LBFGS' then
+    if opt.optim == 'LBFGS' then
 
       -- Perform LBFGS step:
       lbfgsState = lbfgsState or {
@@ -315,7 +442,7 @@ local function train(dataset, opt, model, criterion, epoch,
       logger.messagex(' - nb of iterations: ' .. lbfgsState.nIter)
       logger.messagex(' - nb of function evalutions: ' .. lbfgsState.funcEval)
 
-    elseif opt.optimization == 'ADAM' then
+    elseif opt.optim == 'ADAM' then
       sgdState = sgdState or {
         learningRate = tonumber(opts.learning_rate),-- opt.learningRate,
         momentum = tonumber(opts.momentum), -- opt.momentum,
@@ -323,7 +450,7 @@ local function train(dataset, opt, model, criterion, epoch,
         weightDecay = tonumber(opts.weight_decay),
       }
       optim.adam(feval, parameters, sgdState)
-    elseif opt.optimization == 'ADAGRAD' then
+    elseif opt.optim == 'ADAGRAD' then
       sgdState = sgdState or {
         learningRate = tonumber(opts.learning_rate),-- opt.learningRate,
         momentum = tonumber(opts.momentum), -- opt.momentum,
@@ -331,7 +458,7 @@ local function train(dataset, opt, model, criterion, epoch,
         weightDecay = tonumber(opts.weight_decay),
       }
       optim.adagrad(feval, parameters, sgdState)
-    elseif opt.optimization == 'SGD' then
+    elseif opt.optim == 'SGD' then
       sgdState = sgdState or {
         learningRate = tonumber(opts.learning_rate),-- opt.learningRate,
         momentum = tonumber(opts.momentum), -- opt.momentum,
@@ -339,7 +466,7 @@ local function train(dataset, opt, model, criterion, epoch,
         weightDecay = tonumber(opts.weight_decay),
       }
       optim.sgd(feval, parameters, sgdState)
-    elseif opt.optimization == 'NAG' then
+    elseif opt.optim == 'NAG' then
       sgdState = sgdState or {
         learningRate = tonumber(opts.learning_rate),-- opt.learningRate,
         momentum = tonumber(opts.momentum), -- opt.momentum,
@@ -348,7 +475,8 @@ local function train(dataset, opt, model, criterion, epoch,
       }
       optim.nag(feval, parameters, sgdState)
     else
-      error('unknown optimization method')
+      logger.errx('unknown optimization method: %s', opt.optim)
+      os.exit(1)
     end
   end
 
@@ -363,19 +491,6 @@ local function train(dataset, opt, model, criterion, epoch,
   confusion:zero()
 end
 
-
-local default_opts = {
-  verbose = true,
-  iters = 10,
-  threads = 1,
-  batch_size = 1000,
-  optimization = 'ADAM',
-  learning_rate_decay = 0.001,
-  momentum = 0.1,
-  l1 = 0.0,
-  l2 = 0.0,
-}
-
 local learning_rates = {
   0.01
 }
@@ -383,20 +498,6 @@ local penalty_weights = {
   0
 }
 
-local function override_defaults(def, override)
-  for k,v in pairs(override) do
-    if def[k] then
-      if type(v) == 'table' then
-        override_defaults(def[k], v)
-      else
-        def[k] = v
-      end
-    else
-      def[k] = v
-    end
-  end
-end
-
 local function get_threshold()
   local actions = rspamd_config:get_all_actions()
 
@@ -407,11 +508,27 @@ local function get_threshold()
       or actions['reject']), actions['reject']
 end
 
-return function (args, cfg)
-  opts = default_opts
-  override_defaults(opts, getopt.getopt(args, 'i:'))
+local function handler(args)
+  opts = parser:parse(args)
+  if not opts['log'] then
+    parser:error('no log specified')
+  end
+
+  local _r,err = rspamd_config:load_ucl(opts['config'])
+
+  if not _r then
+    logger.errx('cannot parse %s: %s', opts['config'], err)
+    os.exit(1)
+  end
+
+  _r,err = rspamd_config:parse_rcl({'logging', 'worker'})
+  if not _r then
+    logger.errx('cannot process %s: %s', opts['config'], err)
+    os.exit(1)
+  end
+
   local threshold,reject_score = get_threshold()
-  local logs = rescore_utility.get_all_logs(cfg["logdir"])
+  local logs,messages = rescore_utility.get_all_logs(opts['log'])
 
   if opts['ignore-symbol'] then
     local function add_ignore(s)
@@ -460,15 +577,15 @@ return function (args, cfg)
     end
   end
 
-  if opts['i'] then opts['iters'] = opts['i'] end
-
   local all_symbols = rescore_utility.get_all_symbols(logs, ignore_symbols)
   local original_symbol_scores = rescore_utility.get_all_symbol_scores(rspamd_config,
       ignore_symbols)
 
   -- Display hit frequencies
-  if opts['z'] then
-      local _, all_symbols_stats = rescore_utility.generate_statistics_from_logs(logs, threshold)
+  if opts['freq'] then
+      local _, all_symbols_stats = rescore_utility.generate_statistics_from_logs(logs,
+          messages,
+          threshold)
       local t = {}
       for _, symbol_stats in pairs(all_symbols_stats) do table.insert(t, symbol_stats) end
 
@@ -487,8 +604,8 @@ return function (args, cfg)
                      "NAME", "HITS", "HAM", "HAM%", "SPAM", "SPAM%", "S/O", "OVER%"))
       for _, symbol_stats in pairs(t) do
           logger.message(
-              string.format("%-40s %6d %6d %6.2f %6d %6.2f %6.2f %6.2f", 
-                  symbol_stats.name, 
+              string.format("%-40s %6d %6d %6.2f %6d %6.2f %6.2f %6.2f",
+                  symbol_stats.name,
                   symbol_stats.no_of_hits,
                   symbol_stats.ham_hits,
                   lua_util.round(symbol_stats.ham_percent,2),
@@ -501,7 +618,7 @@ return function (args, cfg)
       end
 
       -- Print file statistics
-      print_stats(logs, threshold)
+      print_stats(logs, messages, threshold)
 
       -- Work out how many symbols weren't seen in the corpus
       local symbols_no_hits = {}
@@ -517,7 +634,7 @@ return function (args, cfg)
           -- Calculate percentage of rules with no hits
           local nhpct = lua_util.round((#symbols_no_hits/total_symbols)*100,2)
           logger.message(
-              string.format('\nFound %s (%-.2f%%) symbols out of %s with no hits in corpus:', 
+              string.format('\nFound %s (%-.2f%%) symbols out of %s with no hits in corpus:',
                             #symbols_no_hits, nhpct, total_symbols
               )
           )
@@ -529,13 +646,13 @@ return function (args, cfg)
       return
   end
 
-  shuffle(logs)
+  shuffle(logs, messages)
   torch.setdefaulttensortype('torch.FloatTensor')
 
-  local train_logs, validation_logs = split_logs(logs, 70)
-  local cv_logs, test_logs = split_logs(validation_logs, 50)
+  local train_logs, validation_logs = split_logs(logs, messages,70)
+  local cv_logs, test_logs = split_logs(validation_logs[1], validation_logs[2], 50)
 
-  local dataset = make_dataset_from_logs(train_logs, all_symbols, reject_score)
+  local dataset = make_dataset_from_logs(train_logs[1], all_symbols, reject_score)
 
   -- Start of perceptron training
   local input_size = #all_symbols
@@ -569,7 +686,8 @@ return function (args, cfg)
             initial_weights)
       end
 
-      local fscore, fps, fns = calculate_fscore_from_weights(cv_logs,
+      local fscore, fps, fns = calculate_fscore_from_weights(cv_logs[1],
+          cv_logs[2],
           all_symbols,
           linear_module.weight[1],
           threshold)
@@ -594,23 +712,23 @@ return function (args, cfg)
 
   new_symbol_scores = stitch_new_scores(all_symbols, new_symbol_scores)
 
-  if cfg["output"] then
-    write_scores(new_symbol_scores, cfg["output"])
+  if opts["output"] then
+    write_scores(new_symbol_scores, opts["output"])
   end
 
-  if cfg["diff"] then
+  if opts["diff"] then
     print_score_diff(new_symbol_scores, original_symbol_scores)
   end
 
   -- Pre-rescore test stats
   logger.message("\n\nPre-rescore test stats\n")
-  test_logs = update_logs(test_logs, original_symbol_scores)
-  print_stats(test_logs, threshold)
+  test_logs[1] = update_logs(test_logs[1], original_symbol_scores)
+  print_stats(test_logs[1], test_logs[2], threshold)
 
   -- Post-rescore test stats
-  test_logs = update_logs(test_logs, new_symbol_scores)
+  test_logs[1] = update_logs(test_logs[1], new_symbol_scores)
   logger.message("\n\nPost-rescore test stats\n")
-  print_stats(test_logs, threshold)
+  print_stats(test_logs[1], test_logs[2], threshold)
 
   logger.messagex('Best fscore=%s, best learning rate=%s, best weight decay=%s',
       best_fscore, best_learning_rate, best_weight_decay)
@@ -630,3 +748,10 @@ return function (args, cfg)
       end
   end
 end
+
+
+return {
+  handler = handler,
+  description = parser._description,
+  name = 'rescore'
+}
\ No newline at end of file
