@@ -1,11 +1,11 @@
 local paths = {}
 
 paths.install_prefix = [[/usr/local/share/rspamd]]
-paths.install_bin = [[/usr/local/share/rspamd/../../../obj/ports/rspamd-1.7.1/rspamd-1.7.1/contrib/torch/torch7]]
+paths.install_bin = [[/usr/local/share/rspamd/../../../obj/ports/rspamd-1.7.9/rspamd-1.7.9/contrib/lua-torch/torch7]]
 paths.install_man = [[/usr/local/share/rspamd/share/man]]
 paths.install_lib = [[/usr/local/share/rspamd//usr/local/share/rspamd/lib]]
 paths.install_share = [[/usr/local/share/rspamd/share]]
-paths.install_include = [[/usr/local/share/rspamd/../../../obj/ports/rspamd-1.7.1/rspamd-1.7.1/contrib/torch/torch7]]
+paths.install_include = [[/usr/local/share/rspamd/../../../obj/ports/rspamd-1.7.9/rspamd-1.7.9/contrib/lua-torch/torch7]]
 paths.install_cmake = [[/usr/local/share/rspamd/share/cmake/torch]]
 
 return paths
