@@ -33,6 +33,8 @@ antivirus {
   clamav {
     # If set force this action if any virus is found (default unset: no action is forced)
     # action = "reject";
+    # If set, then rejection message is set to this value (mention single quotes)
+    # message = '${SCANNER}: virus found: "${VIRUS}"';
     # if `true` only messages with non-image attachments will be checked (default true)
     attachments_only = true;
     # If `max_size` is set, messages > n bytes in size are not scanned
@@ -64,6 +66,8 @@ antivirus {
   return
 end
 
+local default_message = '${SCANNER}: virus found: "${VIRUS}"'
+
 local function match_patterns(default_sym, found, patterns)
   if type(patterns) ~= 'table' then return default_sym end
   if not patterns[1] then
@@ -113,7 +117,10 @@ local function yield_result(task, rule, vname)
       vname = table.concat(vname, '; ')
     end
     task:set_pre_result(rule['action'],
-        string.format('%s: virus found: "%s"', rule['type'], vname))
+        lua_util.template(rule.message or 'Rejected', {
+          SCANNER = rule['type'],
+          VIRUS = vname,
+        }))
   end
 end
 
@@ -125,6 +132,7 @@ local function clamav_config(opts)
     timeout = 15.0,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -162,6 +170,7 @@ local function fprot_config(opts)
     log_clean = false,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -199,6 +208,7 @@ local function sophos_config(opts)
     log_clean = false,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -237,6 +247,7 @@ local function savapi_config(opts)
     timeout = 15.0,
     retransmits = 2,
     cache_expire = 3600, -- expire redis in one hour
+    message = default_message,
   }
 
   for k,v in pairs(opts) do
@@ -266,36 +277,23 @@ local function savapi_config(opts)
   return nil
 end
 
-local function message_not_too_large(task, rule)
+local function message_not_too_large(task, content, rule)
   local max_size = tonumber(rule['max_size'])
   if not max_size then return true end
-  if task:get_size() > max_size then
+  if #content > max_size then
     rspamd_logger.infox("skip %s AV check as it is too large: %s (%s is allowed)",
-      rule.type, task:get_size(), max_size)
+      rule.type, #content, max_size)
     return false
   end
   return true
 end
 
-local function need_av_check(task, rule)
-  if rule['attachments_only'] then
-    for _,p in ipairs(task:get_parts()) do
-      if p:get_filename() and not p:is_image() then
-        return message_not_too_large(task, rule)
-      end
-    end
-
-    rspamd_logger.infox("skip %s AV check as there are no attachments in a message",
-      rule.type)
-
-    return false
-  else
-    return message_not_too_large(task, rule)
-  end
+local function need_av_check(task, content, rule)
+  return message_not_too_large(task, content, rule)
 end
 
-local function check_av_cache(task, rule, fn)
-  local key = task:get_digest()
+local function check_av_cache(task, digest, rule, fn)
+  local key = digest
 
   local function redis_av_cb(err, data)
     if data and type(data) == 'string' then
@@ -334,8 +332,8 @@ local function check_av_cache(task, rule, fn)
   return false
 end
 
-local function save_av_cache(task, rule, to_save)
-  local key = task:get_digest()
+local function save_av_cache(task, digest, rule, to_save)
+  local key = digest
 
   local function redis_set_cb(err)
     -- Do nothing
@@ -367,14 +365,15 @@ local function save_av_cache(task, rule, to_save)
   return false
 end
 
-local function fprot_check(task, rule)
+local function fprot_check(task, content, digest, rule)
   local function fprot_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local scan_id = task:get_queue_id()
     if not scan_id then scan_id = task:get_uid() end
-    local header = string.format('SCAN STREAM %s SIZE %d\n', scan_id, task:get_size())
+    local header = string.format('SCAN STREAM %s SIZE %d\n', scan_id,
+        #content)
     local footer = '\n'
 
     local function fprot_callback(err, data)
@@ -382,21 +381,26 @@ local function fprot_check(task, rule)
         if err == 'IO timeout' then
           if retransmits > 0 then
             retransmits = retransmits - 1
+            -- Select a different upstream!
+            upstream = rule.upstreams:get_upstream_round_robin()
+            addr = upstream:get_addr()
             tcp.request({
               task = task,
               host = addr:to_string(),
               port = addr:get_port(),
               timeout = rule['timeout'],
               callback = fprot_callback,
-              data = { header, task:get_content(), footer },
+              data = { header, content, footer },
               stop_pattern = '\n'
             })
           else
             rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
+            task:insert_result(rule['symbol_fail'], 0.0, 'retransmits exceed')
             upstream:fail()
           end
         else
           rspamd_logger.errx(task, 'failed to scan: %s', err)
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan')
           upstream:fail()
         end
       else
@@ -421,7 +425,7 @@ local function fprot_check(task, rule)
           end
         end
         if cached then
-          save_av_cache(task, rule, cached)
+          save_av_cache(task, digest, rule, cached)
         end
       end
     end
@@ -432,13 +436,13 @@ local function fprot_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = fprot_callback,
-      data = { header, task:get_content(), footer },
+      data = { header, content, footer },
       stop_pattern = '\n'
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, fprot_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, fprot_check_uncached) then
       return
     else
       fprot_check_uncached()
@@ -446,13 +450,13 @@ local function fprot_check(task, rule)
   end
 end
 
-local function clamav_check(task, rule)
+local function clamav_check(task, content, digest, rule)
   local function clamav_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local header = rspamd_util.pack("c9 c1 >I4", "zINSTREAM", "\0",
-      task:get_size())
+      #content)
     local footer = rspamd_util.pack(">I4", 0)
 
     local function clamav_callback(err, data)
@@ -460,31 +464,40 @@ local function clamav_check(task, rule)
         if err == 'IO timeout' then
           if retransmits > 0 then
             retransmits = retransmits - 1
+            -- Select a different upstream!
+            upstream = rule.upstreams:get_upstream_round_robin()
+            addr = upstream:get_addr()
+
             tcp.request({
               task = task,
               host = addr:to_string(),
               port = addr:get_port(),
               timeout = rule['timeout'],
               callback = clamav_callback,
-              data = { header, task:get_content(), footer },
+              data = { header, content, footer },
               stop_pattern = '\0'
             })
           else
             rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
+            task:insert_result(rule['symbol_fail'], 0.0, 'retransmits exceed')
             upstream:fail()
           end
         else
           rspamd_logger.errx(task, 'failed to scan: %s', err)
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan')
           upstream:fail()
         end
       else
         upstream:ok()
         data = tostring(data)
         local cached
+        rspamd_logger.debugm(N, task, '%s [%s]: got reply: %s', rule['symbol'], rule['type'], data)
         if data == 'stream: OK' then
           cached = 'OK'
           if rule['log_clean'] then
             rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
+          else
+            rspamd_logger.debugm(N, task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
           end
         else
           local vname = string.match(data, 'stream: (.+) FOUND')
@@ -493,10 +506,11 @@ local function clamav_check(task, rule)
             cached = vname
           else
             rspamd_logger.errx(task, 'unhandled response: %s', data)
+            task:insert_result(rule['symbol_fail'], 0.0, 'unhandled response')
           end
         end
         if cached then
-          save_av_cache(task, rule, cached)
+          save_av_cache(task, digest, rule, cached)
         end
       end
     end
@@ -507,13 +521,13 @@ local function clamav_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = clamav_callback,
-      data = { header, task:get_content(), footer },
+      data = { header, content, footer },
       stop_pattern = '\0'
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, clamav_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, clamav_check_uncached) then
       return
     else
       clamav_check_uncached()
@@ -521,34 +535,40 @@ local function clamav_check(task, rule)
   end
 end
 
-local function sophos_check(task, rule)
+local function sophos_check(task, content, digest, rule)
   local function sophos_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local protocol = 'SSSP/1.0\n'
-    local streamsize = string.format('SCANDATA %d\n', task:get_size())
+    local streamsize = string.format('SCANDATA %d\n', #content)
     local bye = 'BYE\n'
 
     local function sophos_callback(err, data, conn)
+
       if err then
         if err == 'IO timeout' then
           if retransmits > 0 then
             retransmits = retransmits - 1
+            -- Select a different upstream!
+            upstream = rule.upstreams:get_upstream_round_robin()
+            addr = upstream:get_addr()
             tcp.request({
               task = task,
               host = addr:to_string(),
               port = addr:get_port(),
               timeout = rule['timeout'],
               callback = sophos_callback,
-              data = { protocol, streamsize, task:get_content(), bye }
+              data = { protocol, streamsize, content, bye }
             })
           else
             rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
+            task:insert_result(rule['symbol_fail'], 0.0, 'retransmits exceed')
             upstream:fail()
           end
         else
           rspamd_logger.errx(task, 'failed to scan: %s', err)
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan')
           upstream:fail()
         end
       else
@@ -557,17 +577,32 @@ local function sophos_check(task, rule)
         local vname = string.match(data, 'VIRUS (%S+) ')
         if vname then
           yield_result(task, rule, vname)
-          save_av_cache(task, rule, vname)
+          save_av_cache(task, digest, rule, vname)
         else
           if string.find(data, 'DONE OK') then
             if rule['log_clean'] then
               rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
             end
-            save_av_cache(task, rule, 'OK')
+            save_av_cache(task, digest, rule, 'OK')
+          elseif string.find(data, 'FAIL 0212') then
+            if rule['savdi_report_encrypted'] then
+              rspamd_logger.infox(task, 'Message is ENCRYPTED (0212 SOPHOS_SAVI_ERROR_FILE_ENCRYPTED): %s', data)
+              yield_result(task, rule, "SAVDI_FILE_ENCRYPTED")
+              save_av_cache(task, digest, rule, "SAVDI_FILE_ENCRYPTED")
+            end
+          elseif string.find(data, 'REJ 4') then
+            if rule['savdi_report_oversize'] then
+              rspamd_logger.infox(task, 'Message is OVERSIZED (SSSP reject code 4): %s', data)
+              yield_result(task, digest, rule, "SAVDI_FILE_OVERSIZED")
+              save_av_cache(task, digest, rule, "SAVDI_FILE_OVERSIZED")
+            end
+          elseif string.find(data, 'REJ 1') then
+            rspamd_logger.errx(task, 'SAVDI (Protocol error (REJ 1)): %s', data)
           elseif string.find(data, 'ACC') or string.find(data, 'OK SSSP') then
             conn:add_read(sophos_callback)
           else
             rspamd_logger.errx(task, 'unhandled response: %s', data)
+            task:insert_result(rule['symbol_fail'], 0.0, 'unhandled response')
           end
         end
       end
@@ -579,12 +614,12 @@ local function sophos_check(task, rule)
       port = addr:get_port(),
       timeout = rule['timeout'],
       callback = sophos_callback,
-      data = { protocol, streamsize, task:get_content(), bye }
+      data = { protocol, streamsize, content, bye }
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, sophos_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, sophos_check_uncached) then
       return
     else
       sophos_check_uncached()
@@ -592,7 +627,7 @@ local function sophos_check(task, rule)
   end
 end
 
-local function savapi_check(task, rule)
+local function savapi_check(task, content, digest, rule)
   local function savapi_check_uncached ()
     local upstream = rule.upstreams:get_upstream_round_robin()
     local addr = upstream:get_addr()
@@ -617,7 +652,7 @@ local function savapi_check(task, rule)
         end
 
         yield_result(task, rule, vname)
-        save_av_cache(task, rule, vname)
+        save_av_cache(task, digest, rule, vname)
       end
       if conn then
         conn:close()
@@ -633,7 +668,7 @@ local function savapi_check(task, rule)
         if rule['log_clean'] then
           rspamd_logger.infox(task, '%s: message is clean', rule['type'])
         end
-        save_av_cache(task, rule, 'OK')
+        save_av_cache(task, digest, rule, 'OK')
         conn:add_write(savapi_fin_cb, 'QUIT\n')
 
       -- Terminal response - infected
@@ -683,6 +718,9 @@ local function savapi_check(task, rule)
         if err == 'IO timeout' then
           if retransmits > 0 then
             retransmits = retransmits - 1
+            -- Select a different upstream!
+            upstream = rule.upstreams:get_upstream_round_robin()
+            addr = upstream:get_addr()
             tcp.request({
               task = task,
               host = addr:to_string(),
@@ -694,9 +732,11 @@ local function savapi_check(task, rule)
           else
             rspamd_logger.errx(task, 'failed to scan, maximum retransmits exceed')
             upstream:fail()
+            task:insert_result(rule['symbol_fail'], 0.0, 'retransmits exceed')
           end
         else
           rspamd_logger.errx(task, 'failed to scan: %s', err)
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan')
           upstream:fail()
         end
       else
@@ -720,8 +760,8 @@ local function savapi_check(task, rule)
     })
   end
 
-  if need_av_check(task, rule) then
-    if check_av_cache(task, rule, savapi_check_uncached) then
+  if need_av_check(task, content, rule) then
+    if check_av_cache(task, digest, rule, savapi_check_uncached) then
       return
     else
       savapi_check_uncached()
@@ -750,12 +790,17 @@ local av_types = {
 
 local function add_antivirus_rule(sym, opts)
   if not opts['type'] then
+    rspamd_logger.errx(rspamd_config, 'unknown type for AV rule %s', sym)
     return nil
   end
 
   if not opts['symbol'] then opts['symbol'] = sym end
   local cfg = av_types[opts['type']]
 
+  if not opts['symbol_fail'] then
+    opts['symbol_fail'] = string.upper(opts['type']) .. '_FAIL'
+  end
+
   if not cfg then
     rspamd_logger.errx(rspamd_config, 'unknown antivirus type: %s',
       opts['type'])
@@ -763,6 +808,7 @@ local function add_antivirus_rule(sym, opts)
 
   local rule = cfg.configure(opts)
   rule.type = opts.type
+  rule.symbol_fail = opts.symbol_fail
 
 
   if not rule then
@@ -797,7 +843,21 @@ local function add_antivirus_rule(sym, opts)
   end
 
   return function(task)
-    return cfg.check(task, rule)
+    if rule.attachments_only then
+      local parts = task:get_parts() or {}
+
+      for _,p in ipairs(parts) do
+        if not p:is_image() and not p:is_text() and not p:is_multipart() then
+          local content = p:get_content()
+
+          if content and #content > 0 then
+            cfg.check(task, content, p:get_digest(), rule)
+          end
+        end
+      end
+    else
+      cfg.check(task, task:get_content(), task:get_digest(), rule)
+    end
   end
 end
 
@@ -817,6 +877,12 @@ if opts and type(opts) == 'table' then
           name = m['symbol'],
           callback = cb,
         })
+        rspamd_config:register_symbol({
+          type = 'virtual',
+          name = m['symbol_fail'],
+          parent = id,
+          score = 0.0,
+        })
         has_valid = true
         if type(m['patterns']) == 'table' then
           if m['patterns'][1] then
@@ -826,7 +892,7 @@ if opts and type(opts) == 'table' then
                   rspamd_config:register_symbol({
                     type = 'virtual',
                     name = sym,
-                    parent = id
+                    parent = m['symbol']
                   })
                 end
               end
@@ -866,4 +932,3 @@ if opts and type(opts) == 'table' then
     lua_util.disable_module(N, 'config')
   end
 end
-
