@@ -18,6 +18,9 @@ local rspamd_logger = require 'rspamd_logger'
 local rspamd_http = require "rspamd_http"
 local rspamd_lua_utils = require "lua_util"
 local upstream_list = require "rspamd_upstream_list"
+local lua_util = require "lua_util"
+local ucl = require "ucl"
+
 local N = "clickhouse"
 
 if confighelp then
@@ -29,6 +32,7 @@ local E = {}
 local rows = {}
 local attachment_rows = {}
 local urls_rows = {}
+local emails_rows = {}
 local specific_rows = {}
 local asn_rows = {}
 local symbols_rows = {}
@@ -47,10 +51,13 @@ local settings = {
   dkim_reject_symbols = {'R_DKIM_REJECT'},
   dmarc_allow_symbols = {'DMARC_POLICY_ALLOW'},
   dmarc_reject_symbols = {'DMARC_POLICY_REJECT', 'DMARC_POLICY_QUARANTINE'},
+  stop_symbols = {},
   table = 'rspamd',
   attachments_table = 'rspamd_attachments',
   urls_table = 'rspamd_urls',
+  emails_table = 'rspamd_emails',
   symbols_table = 'rspamd_symbols',
+  asn_table = 'rspamd_asn',
   ipmask = 19,
   ipmask6 = 48,
   full_urls = false,
@@ -59,11 +66,20 @@ local settings = {
   use_https = false,
   use_gzip = true,
   allow_local = false,
+  user = nil,
+  password = nil,
+  no_ssl_verify = false,
+  retention = {
+    enable = false,
+    method = 'detach',
+    period_months = 3,
+    run_every = '7d',
+  }
 }
 
 local clickhouse_schema = {
-rspamd = [[
-CREATE TABLE IF NOT EXISTS rspamd
+table = [[
+CREATE TABLE IF NOT EXISTS ${table}
 (
     Date Date,
     TS DateTime,
@@ -90,8 +106,8 @@ CREATE TABLE IF NOT EXISTS rspamd
 ) ENGINE = MergeTree(Date, (TS, From), 8192)
 ]],
 
-  attachments = [[
-CREATE TABLE IF NOT EXISTS rspamd_attachments (
+attachments_table = [[
+CREATE TABLE IF NOT EXISTS ${attachments_table} (
     Date Date,
     Digest FixedString(32),
     `Attachments.FileName` Array(String),
@@ -101,8 +117,8 @@ CREATE TABLE IF NOT EXISTS rspamd_attachments (
 ) ENGINE = MergeTree(Date, Digest, 8192)
 ]],
 
-  urls = [[
-CREATE TABLE IF NOT EXISTS rspamd_urls (
+urls_table = [[
+CREATE TABLE IF NOT EXISTS ${urls_table} (
     Date Date,
     Digest FixedString(32),
     `Urls.Tld` Array(String),
@@ -110,8 +126,16 @@ CREATE TABLE IF NOT EXISTS rspamd_urls (
 ) ENGINE = MergeTree(Date, Digest, 8192)
 ]],
 
-  asn = [[
-CREATE TABLE IF NOT EXISTS rspamd_asn (
+emails_table = [[
+CREATE TABLE IF NOT EXISTS ${emails_table} (
+    Date Date,
+    Digest FixedString(32),
+    Emails Array(String)
+) ENGINE = MergeTree(Date, Digest, 8192)
+]],
+
+asn_table = [[
+CREATE TABLE IF NOT EXISTS ${asn_table} (
     Date Date,
     Digest FixedString(32),
     ASN String,
@@ -120,8 +144,8 @@ CREATE TABLE IF NOT EXISTS rspamd_asn (
 ) ENGINE = MergeTree(Date, Digest, 8192)
 ]],
 
-  symbols = [[
-CREATE TABLE IF NOT EXISTS rspamd_symbols (
+symbols_table = [[
+CREATE TABLE IF NOT EXISTS ${symbols_table} (
     Date Date,
     Digest FixedString(32),
     `Symbols.Names` Array(String),
@@ -188,6 +212,17 @@ local function clickhouse_urls_row(tname)
   return elt
 end
 
+local function clickhouse_emails_row(tname)
+  local emails_fields = {
+    'Date',
+    'Digest',
+    'Emails',
+  }
+  local elt = string.format('INSERT INTO %s (%s) VALUES ',
+      tname, table.concat(emails_fields, ','))
+  return elt
+end
+
 local function clickhouse_symbols_row(tname)
   local symbols_fields = {
     'Date',
@@ -224,6 +259,10 @@ local function clickhouse_first_row()
     table.insert(urls_rows,
       clickhouse_urls_row(settings['urls_table']))
   end
+  if settings['emails_table'] then
+    table.insert(emails_rows,
+        clickhouse_emails_row(settings['emails_table']))
+  end
   if settings['asn_table'] then
     table.insert(asn_rows,
       clickhouse_asn_row(settings['asn_table']))
@@ -253,15 +292,18 @@ local function clickhouse_send_data(task)
   local upstream = settings.upstream:get_upstream_round_robin()
   local ip_addr = upstream:get_addr():to_string(true)
 
-  local function http_cb(err_message, code, _, _)
-    if code ~= 200 or err_message then
-      rspamd_logger.errx(task, "cannot send data to clickhouse server %s: %s",
-        ip_addr, err_message)
-      upstream:fail()
-    else
-      rspamd_logger.infox(task, "sent %s rows to clickhouse server %s",
-        settings['limit'], ip_addr)
-      upstream:ok()
+  local function gen_http_cb(what, how_many)
+    return function (err_message, code, data, _)
+      if code ~= 200 or err_message then
+        if not err_message then err_message = data end
+        rspamd_logger.errx(task, "cannot send %s data to clickhouse server %s: %s",
+            what, ip_addr, err_message)
+        upstream:fail()
+      else
+        rspamd_logger.infox(task, "sent %s rows of %s to clickhouse server %s",
+            how_many - 1, what, ip_addr)
+        upstream:ok()
+      end
     end
   end
 
@@ -270,10 +312,13 @@ local function clickhouse_send_data(task)
       task = task,
       url = connect_prefix .. ip_addr,
       body = body,
-      callback = http_cb,
+      callback = gen_http_cb('generic data', #rows),
       gzip = settings.use_gzip,
       mime_type = 'text/plain',
       timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
     }) then
      rspamd_logger.errx(task, "cannot send data to clickhouse server %s: cannot make request",
         settings['server'])
@@ -285,9 +330,12 @@ local function clickhouse_send_data(task)
       task = task,
       url = connect_prefix .. ip_addr,
       body = body,
-      callback = http_cb,
+      callback = gen_http_cb('attachments data', #attachment_rows),
       mime_type = 'text/plain',
       timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
     }) then
       rspamd_logger.errx(task, "cannot send attachments to clickhouse server %s: cannot make request",
         settings['server'])
@@ -299,23 +347,46 @@ local function clickhouse_send_data(task)
       task = task,
       url = connect_prefix .. ip_addr,
       body = body,
-      callback = http_cb,
+      callback = gen_http_cb('urls data', #urls_rows),
       mime_type = 'text/plain',
       timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
     }) then
       rspamd_logger.errx(task, "cannot send urls to clickhouse server %s: cannot make request",
         settings['server'])
     end
   end
+  if #emails_rows > 1 then
+    body = table.concat(emails_rows, ' ')
+    if not rspamd_http.request({
+      task = task,
+      url = connect_prefix .. ip_addr,
+      body = body,
+      callback = gen_http_cb('emails data', #emails_rows),
+      mime_type = 'text/plain',
+      timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
+    }) then
+      rspamd_logger.errx(task, "cannot send emails to clickhouse server %s: cannot make request",
+          settings['server'])
+    end
+  end
   if #asn_rows > 1 then
     body = table.concat(asn_rows, ' ')
     if not rspamd_http.request({
       task = task,
       url = connect_prefix .. ip_addr,
       body = body,
-      callback = http_cb,
+      callback = gen_http_cb('asn data', #asn_rows),
       mime_type = 'text/plain',
       timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
     }) then
       rspamd_logger.errx(task, "cannot send asn info to clickhouse server %s: cannot make request",
         settings['server'])
@@ -328,9 +399,12 @@ local function clickhouse_send_data(task)
       task = task,
       url = connect_prefix .. ip_addr,
       body = body,
-      callback = http_cb,
+      callback = gen_http_cb('symbols data', #symbols_rows),
       mime_type = 'text/plain',
       timeout = settings['timeout'],
+      no_ssl_verify = settings.no_ssl_verify,
+      user = settings.user,
+      password = settings.password,
     }) then
       rspamd_logger.errx(task, "cannot send symbols info to clickhouse server %s: cannot make request",
         settings['server'])
@@ -344,9 +418,12 @@ local function clickhouse_send_data(task)
         task = task,
         url = connect_prefix .. ip_addr,
         body = body,
-        callback = http_cb,
+        callback = gen_http_cb('domain specific data ('..k..')', #specific),
         mime_type = 'text/plain',
         timeout = settings['timeout'],
+        no_ssl_verify = settings.no_ssl_verify,
+        user = settings.user,
+        password = settings.password,
       }) then
         rspamd_logger.errx(task, "cannot send data for domain %s to clickhouse server %s: cannot make request",
           k, settings['server'])
@@ -365,6 +442,14 @@ end
 
 local function clickhouse_collect(task)
   if not settings.allow_local and rspamd_lua_utils.is_rspamc_or_controller(task) then return end
+
+  for _,sym in ipairs(settings.stop_symbols) do
+    if task:has_symbol(sym) then
+      rspamd_logger.debugm(N, task, 'skip collection as symbol %s has fired', sym)
+      return
+    end
+  end
+
   local from_domain = ''
   local from_user = ''
   if task:has_from('smtp') then
@@ -561,7 +646,7 @@ local function clickhouse_collect(task)
   local urls_tlds = {}
   local urls_urls = {}
   if task:has_urls(false) then
-    for _,u in ipairs(task:get_urls()) do
+    for _,u in ipairs(task:get_urls(false)) do
       table.insert(urls_tlds, string.format("'%s'", clickhouse_quote(u:get_tld())))
       if settings['full_urls'] then
         table.insert(urls_urls, string.format("'%s'",
@@ -581,6 +666,23 @@ local function clickhouse_collect(task)
     table.insert(urls_rows, elt)
   end
 
+  -- Emails step
+  local emails = {}
+  if task:has_urls(true) then
+    for _,u in ipairs(task:get_emails()) do
+      table.insert(emails, string.format("'%s'", clickhouse_quote(
+          string.format('%s@%s', u:get_user(), u:get_host())
+      )))
+    end
+  end
+
+  if #emails > 0 then
+    elt = string.format("(today(),'%s',[%s])",
+        task:get_digest(),
+        table.concat(emails, ','))
+    table.insert(emails_rows, elt)
+  end
+
   -- ASN information
   if settings['asn_table'] then
     local asn, country, ipnet = '--', '--', '--'
@@ -640,6 +742,7 @@ local function clickhouse_collect(task)
     rows = {}
     attachment_rows = {}
     urls_rows = {}
+    emails_rows = {}
     specific_rows = {}
     asn_rows = {}
     symbols_rows = {}
@@ -647,6 +750,174 @@ local function clickhouse_collect(task)
   end
 end
 
+local function mk_remove_http_cb(upstream, params, ok_cb)
+  local function do_remove_http_cb(err_message, code, data, _)
+    if code ~= 200 or err_message then
+      if not err_message then err_message = data end
+      local ip_addr = upstream:get_addr():to_string(true)
+      rspamd_logger.errx(rspamd_config, "request failed on clickhouse server %s: %s",
+              ip_addr, err_message)
+      upstream:fail()
+    else
+      upstream:ok()
+      if (ok_cb) then
+        rspamd_logger.debugm(N, rspamd_config, "do_remove_http_cb ok: %s, %s, %s, %s", err_message, code, data, _)
+        ok_cb(params.ev_base, params.config, data)
+      end
+    end
+  end
+  return do_remove_http_cb
+end
+
+local function clickhouse_request(upstream, ok_cb, params)
+  rspamd_logger.debugm(N, rspamd_config, "clickhouse_request: %s", params.body)
+
+  params.callback = mk_remove_http_cb(upstream, params, ok_cb)
+  params.gzip = settings.use_gzip
+  params.mime_type = 'text/plain'
+  params.timeout = settings['timeout']
+  params.no_ssl_verify = settings.no_ssl_verify
+  params.user = settings.user
+  params.password = settings.password
+  if not params.url then
+    local ip_addr = upstream:get_addr():to_string(true)
+    params.url = connect_prefix .. ip_addr
+  end
+  return rspamd_http.request(params)
+end
+
+local function do_remove_partition(ev_base, cfg, table_name, partition_id)
+  rspamd_logger.debugm(N, rspamd_config, "removing partition %s.%s", table_name, partition_id)
+  local upstream = settings.upstream:get_upstream_round_robin()
+  local remove_partition_sql = "ALTER TABLE ${table_name} ${remove_method} PARTITION ${partition_id}"
+  local remove_method = (settings.retention.method == 'drop') and 'DROP' or 'DETACH'
+  local sql_params = {
+    ['table_name']     = table_name,
+    ['remove_method']  = remove_method,
+    ['partition_id']   = partition_id
+  }
+
+  local sql = rspamd_lua_utils.template(remove_partition_sql, sql_params)
+
+  local ch_params = {
+    body = sql,
+    ev_base = ev_base,
+    cfg = cfg,
+  }
+
+  if not clickhouse_request(upstream, nil, ch_params) then
+    rspamd_logger.errx(rspamd_config, "cannot send data to clickhouse server %s: cannot make request",
+            settings['server'])
+  end
+end
+
+local function parse_clickhouse_response(ev_base, cfg, data)
+  rspamd_logger.debugm(N, rspamd_config, "got clickhouse response: %s", data)
+  if data == nil then
+    -- clickhouse returned no data (i.e. empty resultset): exiting
+    return
+  end
+  local function parse_string(s)
+    local parser = ucl.parser()
+    local res, err = parser:parse_string(s)
+    if not res then
+      rspamd_logger.errx(rspamd_config, 'Parser error: %s', err)
+      return nil
+    end
+    return parser:get_object()
+  end
+
+  -- iterate over rows
+  local ch_rows = lua_util.str_split(data, "\n")
+  for _, plain_row in pairs(ch_rows) do
+    if plain_row and plain_row:len() > 1 then
+      local parsed_row = parse_string(plain_row)
+      do_remove_partition(rspamd_config, cfg, parsed_row.table, parsed_row.partition_id)
+    end
+  end
+end
+
+
+--[[
+  nil   - file is not writable, do not perform removal
+  0     - it's time to perform removal
+  <int> - how many seconds wait until next run
+]]
+local function get_last_removal_ago()
+  local ts_file = string.format('%s/%s', rspamd_paths['DBDIR'], 'clickhouse_retention_run')
+  local f, err = io.open(ts_file, 'r')
+  local write_file
+  local last_ts
+
+  if err then
+    rspamd_logger.debugm(N, rspamd_config, 'Failed to open %s: %s', ts_file, err)
+  else
+    last_ts = tonumber(f:read('*number'))
+    f:close()
+  end
+
+  local current_ts = os.time()
+
+  if last_ts == nil or (last_ts + settings.retention.period) <= current_ts then
+    write_file, err = io.open(ts_file, 'w')
+    if err then
+      rspamd_logger.errx(rspamd_config, 'Failed to open %s, will not perform retention: %s', ts_file, err)
+      return nil
+    end
+
+    local res
+    res, err = write_file:write(tostring(current_ts))
+    if err or res == nil then
+      rspamd_logger.errx(rspamd_config, 'Failed to write %s, will not perform retention: %s', ts_file, err)
+      return nil
+    end
+    write_file:close()
+    return 0
+  end
+
+  return (last_ts + settings.retention.period) - current_ts
+end
+
+local function clickhouse_remove_old_partitions(cfg, ev_base)
+  local last_time_ago = get_last_removal_ago()
+  if last_time_ago == nil then
+    rspamd_logger.errx(rspamd_config, "Failed to get last run time. Disabling retention")
+    return false
+  elseif last_time_ago ~= 0 then
+    return last_time_ago
+  end
+
+  local upstream = settings.upstream:get_upstream_round_robin()
+  local ip_addr = upstream:get_addr():to_string(true)
+
+  local partition_to_remove_sql = "SELECT distinct partition_id, table FROM system.parts WHERE table in ('${tables}') and max_date < toDate(now() - interval ${month} month);"
+
+  local table_names = {}
+  for table_name,_ in pairs(clickhouse_schema) do
+    table.insert(table_names, settings[table_name])
+  end
+  local tables = table.concat(table_names, "', '")
+  local sql_params = {
+    tables = tables,
+    month  = settings.retention.period_months,
+  }
+  local sql = rspamd_lua_utils.template(partition_to_remove_sql, sql_params)
+
+  local ch_params = {
+    body = sql,
+    url  = string.format("%s%s/?default_format=JSONEachRow", connect_prefix, ip_addr),
+    ev_base = ev_base,
+    config = cfg,
+  }
+  if not clickhouse_request(upstream, parse_clickhouse_response, ch_params) then
+    rspamd_logger.errx(rspamd_config, "cannot send data to clickhouse server %s: cannot make request",
+            settings['server'])
+  end
+
+  -- settings.retention.period is added on initialisation, see below
+  return settings.retention.period
+end
+
 local opts = rspamd_config:get_all_opt('clickhouse')
 if opts then
     for k,v in pairs(opts) do
@@ -667,7 +938,7 @@ if opts then
         settings['server'] or settings['servers'], 8123)
 
       if not settings.upstream then
-        rspamd_logger.errx('cannot parse clickhouse address: %s',
+        rspamd_logger.errx(rspamd_config, 'cannot parse clickhouse address: %s',
             settings['server'] or settings['servers'])
         rspamd_lua_utils.disable_module(N, "config")
         return
@@ -694,17 +965,18 @@ if opts then
           for _,up in ipairs(upstreams) do
             local ip_addr = up:get_addr():to_string(true)
 
-            local function http_cb(err_message, code, _, _)
-              if code ~= 200 or err_message then
-                rspamd_logger.errx(rspamd_config, "cannot create table in clickhouse server %s: %s",
-                    ip_addr, err_message)
-                up:fail()
-              else
-                up:ok()
-              end
-            end
-
             local function send_req(elt, sql)
+              local function http_cb(err_message, code, data, _)
+                if code ~= 200 or err_message then
+                  if not err_message then err_message = data end
+                  rspamd_logger.errx(rspamd_config, "cannot create table %s in clickhouse server %s: %s",
+                      elt, ip_addr, err_message)
+                  up:fail()
+                else
+                  up:ok()
+                end
+              end
+
               if not rspamd_http.request({
                 ev_base = ev_base,
                 config = cfg,
@@ -713,6 +985,9 @@ if opts then
                 callback = http_cb,
                 mime_type = 'text/plain',
                 timeout = settings['timeout'],
+                no_ssl_verify = settings.no_ssl_verify,
+                user = settings.user,
+                password = settings.password,
               }) then
                 rspamd_logger.errx(rspamd_config, "cannot create table %s in clickhouse server %s: cannot make request",
                     elt, ip_addr)
@@ -720,9 +995,33 @@ if opts then
             end
 
             for tab,sql in pairs(clickhouse_schema) do
-              send_req(tab, sql)
+              send_req(tab, rspamd_lua_utils.template(sql, settings))
             end
           end
+
+          if settings.retention.enable and settings.retention.method ~= 'drop' and settings.retention.method ~= 'detach' then
+            rspamd_logger.errx(rspamd_config, "retention.method should be either 'drop' or 'detach' (now: %s). Disabling retention",
+                    settings.retention.method)
+            settings.retention.enable = false
+          end
+          if settings.retention.enable and settings.retention.period_months < 1 or settings.retention.period_months > 1000 then
+            rspamd_logger.errx(rspamd_config, "please, set retention.period_months between 1 and 1000 (now: %s). Disabling retention",
+                    settings.retention.period_months)
+            settings.retention.enable = false
+          end
+          local period = lua_util.parse_time_interval(settings.retention.run_every)
+          if settings.retention.enable and period == nil then
+            rspamd_logger.errx(rspamd_config, "invalid value for retention.run_every (%s). Disabling retention",
+                    settings.retention.run_every)
+            settings.retention.enable = false
+          end
+
+          if settings.retention.enable then
+            settings.retention.period = period
+            rspamd_logger.infox(rspamd_config, "retention will be performed each %s seconds for %s month with method %s",
+                    period, settings.retention.period_months, settings.retention.method)
+            rspamd_config:add_periodic(ev_base, 0, clickhouse_remove_old_partitions, false)
+          end
         end
       end)
     end
