@@ -249,6 +249,9 @@ local function greylist_set(task)
   local action = task:get_metric_action('default')
   local ip = task:get_ip()
 
+  -- Don't do anything if pre-result has been already set
+  if task:has_pre_result() then return end
+
   if settings.greylist_min_score then
     local score = task:get_metric_score('default')[1]
     if score < settings.greylist_min_score then
@@ -256,7 +259,7 @@ local function greylist_set(task)
       if action == 'greylist' then
         -- We are going to accept message
         rspamd_logger.infox(task, 'Downgrading metric action from "greylist" to "no action"')
-        task:set_metric_action('default', 'no action')
+        task:disable_action('greylist')
       end
       return
     end
@@ -266,7 +269,7 @@ local function greylist_set(task)
     if action == 'greylist' then
       -- We are going to accept message
       rspamd_logger.infox(task, 'Downgrading metric action from "greylist" to "no action"')
-      task:set_metric_action('default', 'no action')
+      task:disable_action('greylist')
     end
     return
   end
@@ -276,7 +279,7 @@ local function greylist_set(task)
       if action == 'greylist' then
         -- We are going to accept message
         rspamd_logger.infox(task, 'Downgrading metric action from "greylist" to "no action"')
-        task:set_metric_action('default', 'no action')
+        task:disable_action('greylist')
       end
       return
     end
@@ -320,7 +323,7 @@ local function greylist_set(task)
     if action == 'greylist' then
       -- We are going to accept message
       rspamd_logger.infox(task, 'Downgrading metric action from "greylist" to "no action"')
-      task:set_metric_action('default', 'no action')
+      task:disable_action('greylist')
     end
 
     task:insert_result(settings['symbol'], 0.0, 'pass', is_whitelisted)
