@@ -82,11 +82,8 @@ local settings = {
     app = 2,
     asp = 2,
     bas = 2,
-    cer = 2,
     cnt = 2,
-    crt = 2,
     csh = 2,
-    der = 2,
     diagcab = 2,
     fxp = 2,
     gadget = 2,
@@ -290,7 +287,7 @@ local full_extensions_map = {
   {"csv", "text/csv"},
   {"cur", "application/octet-stream"},
   {"cxx", "text/plain"},
-  {"dat", "application/octet-stream"},
+  {"dat", {"application/octet-stream", "application/ms-tnef"}},
   {"datasource", "application/xml"},
   {"dbproj", "text/plain"},
   {"dcr", "application/x-director"},
@@ -319,7 +316,7 @@ local full_extensions_map = {
   {"dv", "video/x-dv"},
   {"dvi", "application/x-dvi"},
   {"dwf", "drawing/x-dwf"},
-  {"dwg", "application/acad"},
+  {"dwg", {"application/acad", "image/vnd.dwg"}},
   {"dwp", "application/octet-stream"},
   {"dxf", "application/x-dxf" },
   {"dxr", "application/x-director"},
@@ -347,7 +344,7 @@ local full_extensions_map = {
   {"group", "text/x-ms-group"},
   {"gsm", "audio/x-gsm"},
   {"gtar", "application/x-gtar"},
-  {"gz", "application/x-gzip"},
+  {"gz", {"application/gzip", "application/x-gzip"}},
   {"h", "text/plain"},
   {"hdf", "application/x-hdf"},
   {"hdml", "text/x-hdml"},
@@ -406,9 +403,9 @@ local full_extensions_map = {
   {"jfif", "image/pjpeg"},
   {"jnlp", "application/x-java-jnlp-file"},
   {"jpb", "application/octet-stream"},
-  {"jpe", "image/jpeg"},
-  {"jpeg", "image/jpeg"},
-  {"jpg", "image/jpeg"},
+  {"jpe", {"image/jpeg", "image/pjpeg"}},
+  {"jpeg", {"image/jpeg", "image/pjpeg"}},
+  {"jpg", {"image/jpeg", "image/pjpeg"}},
   {"js", "application/javascript"},
   {"json", "application/json"},
   {"jsx", "text/jscript"},
@@ -602,7 +599,7 @@ local full_extensions_map = {
   {"roff", "application/x-troff"},
   {"rpm", "audio/x-pn-realaudio-plugin"},
   {"rqy", "text/x-ms-rqy"},
-  {"rtf", "application/rtf"},
+  {"rtf", {"application/rtf","application/msword"}},
   {"rtx", "text/richtext"},
   {"rvt", "application/octet-stream" },
   {"ruleset", "application/xml"},
@@ -776,7 +773,7 @@ local full_extensions_map = {
   {"xls", {"application/vnd.ms-excel", "application/vnd.ms-office", "application/x-excel", "application/octet-stream"}},
   {"xlsb", "application/vnd.ms-excel.sheet.binary.macroEnabled.12"},
   {"xlsm", "application/vnd.ms-excel.sheet.macroEnabled.12"},
-  {"xlsx", {"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "application/octet-stream"}},
+  {"xlsx", {"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "application/vnd.ms-excel.12", "application/octet-stream"}},
   {"xlt", "application/vnd.ms-excel"},
   {"xltm", "application/vnd.ms-excel.template.macroEnabled.12"},
   {"xltx", "application/vnd.openxmlformats-officedocument.spreadsheetml.template"},
@@ -801,6 +798,7 @@ local full_extensions_map = {
   {"xwd", "image/x-xwindowdump"},
   {"z", "application/x-compress"},
   {"zip", {"application/zip", "application/x-zip-compressed", "application/octet-stream"}},
+  {"zlib", "application/zlib"},
 }
 
 local function check_mime_type(task)
@@ -868,6 +866,13 @@ local function check_mime_type(task)
         if ext2 then
           check_extension(settings['bad_extensions'][ext],
             settings['bad_extensions'][ext2])
+          -- Check for archive cloaking like .zip.gz
+          if settings['archive_extensions'][ext2]
+            -- Exclude multipart archive extensions, e.g. .zip.001
+            and not string.match(ext, '^%d+$')
+          then
+            task:insert_result(settings['symbol_archive_in_archive'], 1.0, string.format(".%s.%s", ext2, ext))
+          end
         else
           check_extension(settings['bad_extensions'][ext], nil)
         end
