@@ -14,6 +14,10 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ]]--
 
+if confighelp then
+  return
+end
+
 -- Multimap is rspamd module designed to define and operate with different maps
 
 local rules = {}
@@ -23,6 +27,7 @@ local util = require "rspamd_util"
 local regexp = require "rspamd_regexp"
 local rspamd_expression = require "rspamd_expression"
 local rspamd_ip = require "rspamd_ip"
+local lua_util = require "lua_util"
 local redis_params
 local fun = require "fun"
 local N = 'multimap'
@@ -75,6 +80,9 @@ local value_types = {
   mempool = {
     get_value = function(val) return val end,
   },
+  symbol_options = {
+    get_value = function(val) return val end,
+  },
 }
 
 local function ip_to_rbl(ip, rbl)
@@ -85,6 +93,9 @@ local function apply_hostname_filter(task, filter, hostname, r)
   if filter == 'tld' then
     local tld = util.get_tld(hostname)
     return tld
+  elseif filter == 'top' then
+    local tld = util.get_tld(hostname)
+    return tld:match('[^.]*$') or tld
   else
     if not r['re_filter'] then
       local pat = string.match(filter, 'tld:regexp:(.+)')
@@ -115,6 +126,9 @@ local function apply_url_filter(task, filter, url, r)
 
   if filter == 'tld' then
     return url:get_tld()
+  elseif filter == 'top' then
+    local tld = url:get_tld()
+    return tld:match('[^.]*$') or tld
   elseif filter == 'full' then
     return url:get_text()
   elseif filter == 'is_phished' then
@@ -202,6 +216,14 @@ local function apply_url_filter(task, filter, url, r)
         return nil
       end
     end
+  elseif string.find(filter, '^template:') then
+    if not r['template'] then
+      r['template'] = string.match(filter, '^template:(.+)')
+    end
+
+    if r['template'] then
+      return lua_util.template(r['template'], url:to_table())
+    end
   end
 
   return url:get_host()
@@ -223,6 +245,11 @@ local function apply_addr_filter(task, filter, input, rule)
     if addr and addr[1] then
       return addr[1]['domain']
     end
+  elseif filter == 'email:domain:tld' then
+    local addr = util.parse_mail_address(input, task:get_mempool())
+    if addr and addr[1] then
+      return util.get_tld(addr[1]['domain'])
+    end
   elseif filter == 'email:name' then
     local addr = util.parse_mail_address(input, task:get_mempool())
     if addr and addr[1] then
@@ -335,6 +362,7 @@ local multimap_filters = {
   from = apply_addr_filter,
   rcpt = apply_addr_filter,
   helo = apply_hostname_filter,
+  symbol_options = apply_regexp_filter,
   header = apply_addr_filter,
   url = apply_url_filter,
   filename = apply_filename_filter,
@@ -343,6 +371,8 @@ local multimap_filters = {
   --content = apply_content_filter, -- Content filters are special :(
 }
 
+local multimap_grammar
+
 local function multimap_callback(task, rule)
   local pre_filter = rule['prefilter']
 
@@ -366,17 +396,32 @@ local function multimap_callback(task, rule)
       end
       ret = r['cdb']:lookup(srch)
     elseif r['redis_key'] then
-      local srch = value
-      if r['type'] == 'ip' then
-        srch = value:to_string()
+      local srch = {value}
+      local cmd = 'HGET'
+      if r['type'] == 'ip' or (r['type'] == 'received' and
+        (r['filter'] == 'real_ip' or r['filter'] == 'from_ip' or not r['filter'])) then
+        srch = {value:to_string()}
+        cmd = 'HMGET'
+        local maxbits = 128
+        local minbits = 32
+        if value:get_version() == 4 then
+            maxbits = 32
+            minbits = 8
+        end
+        for i=maxbits,minbits,-1 do
+            local nip = value:apply_mask(i):to_string() .. "/" .. i
+            table.insert(srch, nip)
+        end
       end
+
+      table.insert(srch, 1, r['redis_key'])
       ret = rspamd_redis_make_request(task,
         redis_params, -- connect params
         r['redis_key'], -- hash key
         false, -- is write
         redis_map_cb, --callback
-        'HGET', -- command
-        {r['redis_key'], srch} -- arguments
+        cmd, -- command
+        srch -- arguments
       )
 
       return ret
@@ -396,36 +441,39 @@ local function multimap_callback(task, rule)
   local function parse_ret(parse_rule, p_ret)
     if p_ret and type(p_ret) == 'string' then
       local lpeg = require "lpeg"
+
+      if not multimap_grammar then
       local number = {}
 
-      local digit = lpeg.R("09")
-      number.integer =
+        local digit = lpeg.R("09")
+        number.integer =
         (lpeg.S("+-") ^ -1) *
-        (digit   ^  1)
+                (digit   ^  1)
 
-      -- Matches: .6, .899, .9999873
-      number.fractional =
+        -- Matches: .6, .899, .9999873
+        number.fractional =
         (lpeg.P(".")   ) *
-        (digit ^ 1)
+                (digit ^ 1)
 
-      -- Matches: 55.97, -90.8, .9
-      number.decimal =
+        -- Matches: 55.97, -90.8, .9
+        number.decimal =
         (number.integer *              -- Integer
-        (number.fractional ^ -1)) +    -- Fractional
-        (lpeg.S("+-") * number.fractional)  -- Completely fractional number
+                (number.fractional ^ -1)) +    -- Fractional
+                (lpeg.S("+-") * number.fractional)  -- Completely fractional number
 
-      local sym_start = lpeg.R("az", "AZ") + lpeg.S("_")
-      local sym_elt = sym_start + lpeg.R("09")
-      local symbol = sym_start * sym_elt ^0
-      local symbol_cap = lpeg.Cg(symbol, 'symbol')
-      local score_cap = lpeg.Cg(number.decimal, 'score')
-      local symscore_cap = (symbol_cap * lpeg.P(":") * score_cap)
-      local grammar = symscore_cap + symbol_cap + score_cap
-      local parser = lpeg.Ct(grammar)
-      local tbl = parser:match(p_ret)
+        local sym_start = lpeg.R("az", "AZ") + lpeg.S("_")
+        local sym_elt = sym_start + lpeg.R("09")
+        local symbol = sym_start * sym_elt ^0
+        local symbol_cap = lpeg.Cg(symbol, 'symbol')
+        local score_cap = lpeg.Cg(number.decimal, 'score')
+        local symscore_cap = (symbol_cap * lpeg.P(":") * score_cap)
+        local grammar = symscore_cap + symbol_cap + score_cap
+        multimap_grammar = lpeg.Ct(grammar)
+      end
+      local tbl = multimap_grammar:match(p_ret)
 
       if tbl then
-        local sym = nil
+        local sym
         local score = 1.0
 
         if tbl['symbol'] then
@@ -455,13 +503,26 @@ local function multimap_callback(task, rule)
   local function match_rule(r, value)
     local function rule_callback(result)
       if result then
+        if type(result) == 'table' then
+          for _,rs in ipairs(result) do
+            if type(rs) ~= 'userdata' then
+              rule_callback(rs)
+            end
+          end
+          return
+        end
         local _,symbol,score = parse_ret(r, result)
-        if symbol and r['symbols_set'] then
-          if not r['symbols_set'][symbol] then
-            rspamd_logger.infox(task, 'symbol %s is not registered for map %s, ' ..
-              'replace it with just %s',
-              symbol, r['symbol'], r['symbol'])
-            symbol = r['symbol']
+        local forced = false
+        if symbol then
+          if r['symbols_set'] then
+            if not r['symbols_set'][symbol] then
+              rspamd_logger.infox(task, 'symbol %s is not registered for map %s, ' ..
+                  'replace it with just %s',
+                  symbol, r['symbol'], r['symbol'])
+              symbol = r['symbol']
+            end
+          else
+            forced = true
           end
         else
           symbol = r['symbol']
@@ -469,9 +530,9 @@ local function multimap_callback(task, rule)
 
         local opt = value_types[r['type']].get_value(value)
         if opt then
-          task:insert_result(symbol, score, opt)
+          task:insert_result(forced, symbol, score, opt)
         else
-          task:insert_result(symbol, score)
+          task:insert_result(forced, symbol, score)
         end
 
         if pre_filter then
@@ -519,8 +580,11 @@ local function multimap_callback(task, rule)
 
   local function match_addr(r, addr)
     match_list(r, addr, {'addr'})
-    match_list(r, addr, {'domain'})
-    match_list(r, addr, {'user'})
+
+    if not r.filter then
+      match_list(r, addr, {'domain'})
+      match_list(r, addr, {'user'})
+    end
   end
 
   local function match_url(r, url)
@@ -536,7 +600,12 @@ local function multimap_callback(task, rule)
   end
 
   local function match_received_header(r, pos, total, h)
+    local use_tld = false
     local filter = r['filter'] or 'real_ip'
+    if filter:match('^tld:') then
+      filter = filter:sub(5)
+      use_tld = true
+    end
     local v = h[filter]
     if v then
       local min_pos = tonumber(r['min_pos'])
@@ -567,6 +636,21 @@ local function multimap_callback(task, rule)
           end
         end
       end
+      local match_flags = r['flags']
+      local nmatch_flags = r['nflags']
+      if match_flags or nmatch_flags then
+        local got_flags = h['flags']
+        if match_flags then
+          for _, flag in ipairs(match_flags) do
+            if not got_flags[flag] then return end
+          end
+        end
+        if nmatch_flags then
+          for _, flag in ipairs(nmatch_flags) do
+            if got_flags[flag] then return end
+          end
+        end
+      end
       if filter == 'real_ip' or filter == 'from_ip' then
         if type(v) == 'string' then
           v = rspamd_ip.from_string(v)
@@ -575,6 +659,9 @@ local function multimap_callback(task, rule)
           match_rule(r, v)
         end
       else
+        if use_tld and type(v) == 'string' then
+          v = util.get_tld(v)
+        end
         match_rule(r, v)
       end
     end
@@ -636,19 +723,32 @@ local function multimap_callback(task, rule)
       end
     end,
     header = function()
-      local hv = task:get_header_full(rule['header'])
-      match_list(rule, hv, {'decoded'})
+      if type(rule['header']) == 'table' then
+        for _,rh in ipairs(rule['header']) do
+          local hv = task:get_header_full(rh)
+          match_list(rule, hv, {'decoded'})
+        end
+      else
+        local hv = task:get_header_full(rule['header'])
+        match_list(rule, hv, {'decoded'})
+      end
     end,
     rcpt = function()
       if task:has_recipients('smtp') then
         local rcpts = task:get_recipients('smtp')
         match_addr(rule, rcpts)
+      elseif task:has_recipients('mime') then
+        local rcpts = task:get_recipients('mime')
+        match_addr(rule, rcpts)
       end
     end,
     from = function()
       if task:has_from('smtp') then
         local from = task:get_from('smtp')
         match_addr(rule, from)
+      elseif task:has_from('mime') then
+        local from = task:get_from('mime')
+        match_addr(rule, from)
       end
     end,
     helo = function()
@@ -660,6 +760,7 @@ local function multimap_callback(task, rule)
     url = function()
       if task:has_urls() then
         local msg_urls = task:get_urls()
+
         for _,url in ipairs(msg_urls) do
           match_url(rule, url)
         end
@@ -687,7 +788,7 @@ local function multimap_callback(task, rule)
     end,
     hostname = function()
       local hostname = task:get_hostname()
-      if hostname and hostname ~= 'unknown' then
+      if hostname then
         match_hostname(rule, hostname)
       end
     end,
@@ -709,12 +810,20 @@ local function multimap_callback(task, rule)
         match_rule(rule, var)
       end
     end,
+    symbol_options = function()
+      local sym = task:get_symbol(rule['target_symbol'])
+      if sym and sym[1].options then
+        for _, o in ipairs(sym[1].options) do
+          match_rule(rule, o)
+        end
+      end
+    end,
     received = function()
       local hdrs = task:get_received_headers()
       if hdrs and hdrs[1] then
         if not rule['artificial'] then
           hdrs = fun.filter(function(h)
-            return not h['artificial']
+            return not h['flags']['artificial']
           end, hdrs):totable()
         end
         for pos, h in ipairs(hdrs) do
@@ -788,8 +897,8 @@ local function add_multimap_rule(key, newrule)
   else
     if type(newrule['map']) == 'string' then
       local map = urls[newrule['map']]
-      if map and map['type'] == newrule['type']
-        and map['regexp'] == newrule['regexp'] then
+      if map and map['regexp'] == newrule['regexp'] and
+          map['glob'] == newrule['glob'] then
         if newrule['type'] == 'ip' then
           newrule['radix'] = map['map']
         else
@@ -821,6 +930,16 @@ local function add_multimap_rule(key, newrule)
             newrule['map'])
         end
       elseif newrule['type'] == 'received' then
+        if type(newrule['flags']) == 'table' and newrule['flags'][1] then
+          newrule['flags'] = newrule['flags']
+        elseif type(newrule['flags']) == 'string' then
+          newrule['flags'] = {newrule['flags']}
+        end
+        if type(newrule['nflags']) == 'table' and newrule['nflags'][1] then
+          newrule['nflags'] = newrule['nflags']
+        elseif type(newrule['nflags']) == 'string' then
+          newrule['nflags'] = {newrule['nflags']}
+        end
         local filter = newrule['filter'] or 'real_ip'
         if filter == 'real_ip' or filter == 'from_ip' then
           newrule['radix'] = rspamd_config:add_map ({
@@ -838,6 +957,12 @@ local function add_multimap_rule(key, newrule)
               description = newrule['description'],
               type = 'regexp'
             })
+          elseif newrule['glob'] then
+            newrule['hash'] = rspamd_config:add_map ({
+              url = newrule['map'],
+              description = newrule['description'],
+              type = 'glob'
+            })
           else
             newrule['hash'] = rspamd_config:add_map ({
               url = newrule['map'],
@@ -863,6 +988,7 @@ local function add_multimap_rule(key, newrule)
         or newrule['type'] == 'rcpt'
         or newrule['type'] == 'from'
         or newrule['type'] == 'helo'
+        or newrule['type'] == 'symbol_options'
         or newrule['type'] == 'filename'
         or newrule['type'] == 'url'
         or newrule['type'] == 'content'
@@ -876,6 +1002,12 @@ local function add_multimap_rule(key, newrule)
             description = newrule['description'],
             type = 'regexp'
           })
+        elseif newrule['glob'] then
+          newrule['hash'] = rspamd_config:add_map ({
+            url = newrule['map'],
+            description = newrule['description'],
+            type = 'glob'
+          })
         else
           newrule['hash'] = rspamd_config:add_map ({
             url = newrule['map'],
@@ -909,6 +1041,9 @@ local function add_multimap_rule(key, newrule)
   end
 
   if ret then
+    if newrule['type'] == 'symbol_options' then
+      rspamd_config:register_dependency(newrule['symbol'], newrule['target_symbol'])
+    end
     if newrule['require_symbols'] and not newrule['prefilter'] then
       local atoms = {}
 
@@ -962,6 +1097,8 @@ if opts and type(opts) == 'table' then
       if not rule then
         rspamd_logger.errx(rspamd_config, 'cannot add rule: "'..k..'"')
       else
+        rspamd_logger.infox(rspamd_config, 'added multimap rule: %s (%s)',
+            k, rule.type)
         table.insert(rules, rule)
       end
     end
@@ -1013,4 +1150,8 @@ if opts and type(opts) == 'table' then
     })
   end,
   fun.filter(function(r) return r['prefilter'] end, rules))
+
+  if #rules == 0 then
+    lua_util.disable_module(N, "config")
+  end
 end
