@@ -14,14 +14,21 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ]]--
 
+if confighelp then
+  return
+end
+
 -- This plugin implements user dynamic settings
 -- Settings documentation can be found here:
 -- https://rspamd.com/doc/configuration/settings.html
 
 local rspamd_logger = require "rspamd_logger"
+local rspamd_maps = require "lua_maps"
+local lua_squeeze = require "lua_squeeze_rules"
 local redis_params
 
 local settings = {}
+local N = "settings"
 local settings_ids = {}
 local settings_initialized = false
 local max_pri = 0
@@ -30,8 +37,46 @@ local rspamd_regexp = require "rspamd_regexp"
 local ucl = require "ucl"
 local fun = require "fun"
 
--- Checks for overrided settings within query params and returns 'true' if
--- settings are overrided
+local function apply_settings(task, to_apply)
+  task:set_settings(to_apply)
+  task:cache_set('settings', to_apply)
+  lua_squeeze.handle_settings(task, to_apply)
+
+  if to_apply['add_headers'] or to_apply['remove_headers'] then
+    local rep = {
+      add_headers = to_apply['add_headers'] or {},
+      remove_headers = to_apply['remove_headers'] or {},
+    }
+    task:set_rmilter_reply(rep)
+  end
+
+  if to_apply.flags and type(to_apply.flags) == 'table' then
+    for _,fl in ipairs(to_apply.flags) do
+      task:set_flag(fl)
+    end
+  end
+
+  if to_apply.symbols then
+    -- Add symbols, specified in the settings
+    if #to_apply.symbols > 0 then
+      fun.each(function(val)
+        task:insert_result(val, 1.0)
+      end,
+          fun.filter(function(elt) return type(elt) == 'string' end,
+              to_apply.symbols))
+    else
+      -- Object like symbols
+      fun.each(function(k, val)
+        task:insert_result(k, val.score or 1.0, val.options or {})
+      end,
+          fun.filter(function(_, elt) return type(elt) == 'table' end,
+              to_apply.symbols))
+    end
+  end
+end
+
+-- Checks for overridden settings within query params and returns 'true' if
+-- settings are overridden
 local function check_query_settings(task)
   -- Try 'settings' attribute
   local query_set = task:get_request_header('settings')
@@ -39,7 +84,8 @@ local function check_query_settings(task)
     local parser = ucl.parser()
     local res,err = parser:parse_string(tostring(query_set))
     if res then
-      task:set_settings(parser:get_object())
+      local settings_obj = parser:get_object()
+      apply_settings(task, settings_obj)
 
       return true
     else
@@ -66,8 +112,7 @@ local function check_query_settings(task)
         nset['default']['actions']['add header'] = ss
       end
 
-      task:set_settings(nset)
-
+      apply_settings(task, nset)
       return true
     end
   end
@@ -78,9 +123,8 @@ local function check_query_settings(task)
     local id_str = tostring(settings_id)
     local elt = settings_ids[id_str]
     if elt and elt['apply'] then
-      task:set_settings(elt['apply'])
+      apply_settings(task, elt['apply'])
       rspamd_logger.infox(task, "applying settings id %s", id_str)
-
       return true
     end
   end
@@ -93,17 +137,17 @@ local function check_settings(task)
   local function check_addr_setting(rule, addr)
     local function check_specific_addr(elt)
       if rule['name'] then
-        if rule['name']:lower() == elt['addr']:lower() then
+        if rspamd_maps.rspamd_maybe_check_map(rule['name'], elt['addr']) then
           return true
         end
       end
       if rule['user'] then
-        if rule['user']:lower() == elt['user']:lower() then
+        if rspamd_maps.rspamd_maybe_check_map(rule['user'], elt['user']) then
           return true
         end
       end
       if rule['domain'] and elt['domain'] then
-        if rule['domain']:lower() == elt['domain']:lower() then
+        if rspamd_maps.rspamd_maybe_check_map(rule['domain'], elt['domain']) then
           return true
         end
       end
@@ -125,13 +169,19 @@ local function check_settings(task)
   end
 
   local function check_ip_setting(rule, ip)
-    if rule[2] ~= 0 then
-      local nip = ip:apply_mask(rule[2])
-      if nip and nip:to_string() == rule[1]:to_string() then
+    if not rule[2] then
+      if rspamd_maps.rspamd_maybe_check_map(rule[1], ip:to_string()) then
+        return true
+      end
+    else
+      if rule[2] ~= 0 then
+        local nip = ip:apply_mask(rule[2])
+        if nip and nip:to_string() == rule[1]:to_string() then
+          return true
+        end
+      elseif ip:to_string() == rule[1]:to_string() then
         return true
       end
-    elseif ip:to_string() == rule[1]:to_string() then
-      return true
     end
 
     return false
@@ -151,7 +201,7 @@ local function check_settings(task)
     end
 
     if rule['ip'] then
-      if not ip then
+      if not ip or not ip:is_valid() then
         return nil
       end
       for _, i in ipairs(rule['ip']) do
@@ -238,6 +288,29 @@ local function check_settings(task)
       end
     end
 
+    if rule['header'] then
+      for _, e in ipairs(rule['header']) do
+        for k, v in pairs(e) do
+          for _, p in ipairs(v) do
+            local h = task:get_header(k)
+            res = (h and p:match(h))
+            if res then
+              break
+            end
+          end
+          if res then
+            break
+          end
+        end
+        if res then
+          break
+        end
+      end
+      if not res then
+        return nil
+      end
+    end
+
     if res then
       if rule['whitelist'] then
         rule['apply'] = {whitelist = true}
@@ -289,7 +362,7 @@ local function check_settings(task)
           rspamd_logger.infox(task, "<%1> apply settings according to rule %2",
             task:get_message_id(), s.name)
           if rule['apply'] then
-            task:set_settings(rule['apply'])
+            apply_settings(task, rule['apply'])
             applied = true
           end
           if rule['symbols'] then
@@ -351,8 +424,8 @@ local function process_settings_table(tbl)
             out[1] = res
             out[2] = 0
           else
-            rspamd_logger.errx(rspamd_config, "bad IP address: " .. ip)
-            return nil
+            -- It can still be a map
+            out[1] = res
           end
         else
           local res = rspamd_ip.from_string(string.sub(ip, 1, slash - 1))
@@ -380,29 +453,34 @@ local function process_settings_table(tbl)
           table.insert(out, process_addr(v))
         end
       elseif type(addr) == "string" then
-        local start = string.sub(addr, 1, 1)
-        if start == '/' then
-          -- It is a regexp
-          local re = rspamd_regexp.create(addr)
-          if re then
-            out['regexp'] = re
-          else
-            rspamd_logger.errx(rspamd_config, "bad regexp: " .. addr)
-            return nil
-          end
-
-        elseif start == '@' then
-          -- It is a domain if form @domain
-          out['domain'] = string.sub(addr, 2)
+        if string.sub(addr, 1, 4) == "map:" then
+          -- It is map, don't apply any extra logic
+          out['name'] = addr
         else
-          -- Check user@domain parts
-          local at = string.find(addr, '@')
-          if at then
-            -- It is full address
-            out['name'] = addr
+          local start = string.sub(addr, 1, 1)
+          if start == '/' then
+            -- It is a regexp
+            local re = rspamd_regexp.create(addr)
+            if re then
+              out['regexp'] = re
+            else
+              rspamd_logger.errx(rspamd_config, "bad regexp: " .. addr)
+              return nil
+            end
+
+          elseif start == '@' then
+            -- It is a domain if form @domain
+            out['domain'] = string.sub(addr, 2)
           else
-            -- It is a user
-            out['user'] = addr
+            -- Check user@domain parts
+            local at = string.find(addr, '@')
+            if at then
+              -- It is full address
+              out['name'] = addr
+            else
+              -- It is a user
+              out['user'] = addr
+            end
           end
         end
       else
@@ -471,6 +549,35 @@ local function process_settings_table(tbl)
       end
       out['request_header'] = rho
     end
+    if elt['header'] then
+      if not elt['header'][1] and next(elt['header']) then
+        elt['header'] = {elt['header']}
+      end
+      for _, e in ipairs(elt['header']) do
+        local rho = {}
+        for k, v in pairs(e) do
+          if type(v) ~= 'table' then
+            v = {v}
+          end
+          for _, r in ipairs(v) do
+            local re = rspamd_regexp.get_cached(r)
+            if not re then
+              re = rspamd_regexp.create_cached(r)
+            end
+            if re then
+              if not out['header'] then out['header'] = {} end
+              if rho[k] then
+                table.insert(rho[k], re)
+              else
+                rho[k] = {re}
+              end
+            end
+          end
+        end
+        if not out['header'] then out['header'] = {} end
+        table.insert(out['header'], rho)
+      end
+    end
 
     -- Now we must process actions
     if elt['symbols'] then out['symbols'] = elt['symbols'] end
@@ -554,36 +661,51 @@ local function gen_redis_callback(handler, id)
     local key = handler(task)
 
     local function redis_settings_cb(err, data)
-      if not err and type(data) == 'string' then
-        local parser = ucl.parser()
-        local res,ucl_err = parser:parse_string(data)
-        if not res then
-          rspamd_logger.warnx(rspamd_config, 'cannot parse settings from redis: %s',
-            ucl_err)
-        else
-          local obj = parser:get_object()
-          rspamd_logger.infox(task, "<%1> apply settings according to redis rule %2",
-            task:get_message_id(), id)
-          task:set_settings(obj)
+      if not err and type(data) == 'table' then
+        for _, d in ipairs(data) do
+          if type(d) == 'string' then
+            local parser = ucl.parser()
+            local res,ucl_err = parser:parse_string(d)
+            if not res then
+              rspamd_logger.warnx(rspamd_config, 'cannot parse settings from redis: %s',
+                ucl_err)
+            else
+              local obj = parser:get_object()
+              rspamd_logger.infox(task, "<%1> apply settings according to redis rule %2",
+                task:get_message_id(), id)
+              apply_settings(task, obj)
+              break
+            end
+          end
         end
+      elseif err then
+        rspamd_logger.errx(task, 'Redis error: %1', err)
       end
     end
 
     if not key then
-      rspamd_logger.errx(rspamd_config, 'Cannot execute handler number %s', id)
+      rspamd_logger.debugm(N, task, 'handler number %s returned nil', id)
       return
     end
 
+    local keys
+    if type(key) == 'table' then
+      keys = key
+    else
+      keys = {key}
+    end
+    key = keys[1]
+
     local ret,_,_ = rspamd_redis_make_request(task,
       redis_params, -- connect params
       key, -- hash key
       false, -- is write
       redis_settings_cb, --callback
-      'GET', -- command
-      {key} -- arguments
+      'MGET', -- command
+      keys -- arguments
     )
     if not ret then
-      rspamd_logger.errx(task, 'Redis GET failed: %s', ret)
+      rspamd_logger.errx(task, 'Redis MGET failed: %s', ret)
     end
   end
 end
@@ -617,9 +739,10 @@ if redis_section then
   fun.each(function(id, h)
     rspamd_config:register_symbol({
       name = 'REDIS_SETTINGS' .. tostring(id),
-      type = 'prefilter',
+      type = 'prefilter,nostat',
       callback = gen_redis_callback(h, id),
-      priority = 10
+      priority = 10,
+      flags = 'empty',
     })
   end, redis_key_handlers)
 end
@@ -637,7 +760,8 @@ end
 
 rspamd_config:register_symbol({
   name = 'SETTINGS_CHECK',
-  type = 'prefilter',
+  type = 'prefilter,nostat',
   callback = check_settings,
-  priority = 10
+  priority = 10,
+  flags = 'empty',
 })
