@@ -14,6 +14,10 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ]]--
 
+if confighelp then
+  return
+end
+
 -- This plugin is intended to read and parse spamassassin rules with regexp
 -- rules. SA plugins or statistics are not supported
 
@@ -25,6 +29,7 @@ local rspamd_regexp = require "rspamd_regexp"
 local rspamd_expression = require "rspamd_expression"
 local rspamd_trie = require "rspamd_trie"
 local util = require "rspamd_util"
+local lua_util = require "lua_util"
 local fun = require "fun"
 
 -- Known plugins
@@ -37,9 +42,10 @@ local known_plugins = {
   'Mail::SpamAssassin::Plugin::BodyEval',
   'Mail::SpamAssassin::Plugin::MIMEHeader',
   'Mail::SpamAssassin::Plugin::WLBLEval',
+  'Mail::SpamAssassin::Plugin::HTMLEval',
 }
 
--- Table that replaces SA symbol with rspamd equialent
+-- Table that replaces SA symbol with rspamd equivalent
 -- Used for dependency resolution
 local symbols_replacements = {
   -- SPF replacements
@@ -146,10 +152,6 @@ local function replace_symbol(s)
   return rspamd_symbol, true
 end
 
-local function trim(s)
-  return s:match "^%s*(.-)%s*$"
-end
-
 local ffi
 if type(jit) == 'table' then
   ffi = require("ffi")
@@ -355,6 +357,7 @@ local function gen_eval_rule(arg)
                 if r:match(h) then
                   return 1
                 end
+                return 0
               else
                 rspamd_logger.infox(rspamd_config, 'cannot create regexp %1', re)
                 return 0
@@ -520,6 +523,24 @@ local function gen_eval_rule(arg)
         return 0
       end
     },
+    {
+      'html_tag_exists',
+      function(task, remain)
+        local tp = task:get_text_parts()
+
+        for _,p in ipairs(tp) do
+          if p:is_html() then
+            local hc = p:get_html()
+
+            if hc:has_tag(remain) then
+              return 1
+            end
+          end
+        end
+
+        return 0
+      end
+    }
   }
 
   for _,f in ipairs(eval_funcs) do
@@ -677,12 +698,16 @@ local function process_sa_conf(f)
   local if_nested = 0
   for l in f:lines() do
     (function ()
-    l = trim(l)
+    l = lua_util.rspamd_str_trim(l)
+    -- Replace bla=~/re/ with bla =~ /re/ (#2372)
+    l = l:gsub('([^%s])%s*([=!]~)%s*([^%s])', '%1 %2 %3')
 
     if string.len(l) == 0 or string.sub(l, 1, 1) == '#' then
       return
     end
 
+    -- Unbalanced if/endif
+    if if_nested < 0 then if_nested = 0 end
     if skip_to_endif then
       if string.match(l, '^endif') then
         if_nested = if_nested - 1
@@ -692,6 +717,9 @@ local function process_sa_conf(f)
         end
       elseif string.match(l, '^if') then
         if_nested = if_nested + 1
+      elseif string.match(l, '^else') then
+        -- Else counterpart for if
+        skip_to_endif = false
       end
       return
     else
@@ -703,8 +731,8 @@ local function process_sa_conf(f)
             return false
             end, known_plugins) then
           skip_to_endif = true
-          if_nested = 1
         end
+        if_nested = if_nested + 1
       elseif string.match(l, '^if !plugin%(') then
          local pname = string.match(l, '^if !plugin%(([A-Za-z:]+)%)')
          if fun.any(function(pl)
@@ -712,12 +740,17 @@ local function process_sa_conf(f)
            return false
          end, known_plugins) then
            skip_to_endif = true
-           if_nested = 1
          end
+         if_nested = if_nested + 1
       elseif string.match(l, '^if') then
         -- Unknown if
         skip_to_endif = true
-        if_nested = 1
+        if_nested = if_nested + 1
+      elseif string.match(l, '^else') then
+        -- Else counterpart for if
+        skip_to_endif = true
+      elseif string.match(l, '^endif') then
+        if_nested = if_nested - 1
       end
     end
 
@@ -975,7 +1008,10 @@ local function process_sa_conf(f)
       cur_rule['type'] = 'meta'
       cur_rule['symbol'] = words[2]
       cur_rule['meta'] = words_to_re(words, 2)
-      if cur_rule['meta'] and cur_rule['symbol'] then valid_rule = true end
+      if cur_rule['meta'] and cur_rule['symbol']
+        and cur_rule['meta'] ~= '0' then
+          valid_rule = true
+      end
     elseif words[1] == "describe" and valid_rule then
       cur_rule['description'] = words_to_re(words, 2)
     elseif words[1] == "score" then
@@ -1610,4 +1646,6 @@ end
 
 if has_rules then
   post_process()
+else
+  lua_util.disable_module(N, "config")
 end
