@@ -34,6 +34,26 @@ reconf['SUBJECT_NEEDS_ENCODING'] = {
   group = 'header'
 }
 
+local from_encoded_b64 = 'From=/=\\?\\S+\\?B\\?/iX'
+local from_encoded_qp = 'From=/=\\?\\S+\\?Q\\?/iX'
+local raw_from_needs_mime = 'From=/[\\x00-\\x08\\x0b\\x0c\\x0e-\\x1f\\x7f-\\xff]/X'
+reconf['FROM_NEEDS_ENCODING'] = {
+  re = string.format('!(%s) & !(%s) & (%s)', from_encoded_b64, from_encoded_qp, raw_from_needs_mime),
+  score = 1.0,
+  description = 'From header needs encoding',
+  group = 'header'
+}
+
+local to_encoded_b64 = 'To=/=\\?\\S+\\?B\\?/iX'
+local to_encoded_qp = 'To=/=\\?\\S+\\?Q\\?/iX'
+local raw_to_needs_mime = 'To=/[\\x00-\\x08\\x0b\\x0c\\x0e-\\x1f\\x7f-\\xff]/X'
+reconf['TO_NEEDS_ENCODING'] = {
+  re = string.format('!(%s) & !(%s) & (%s)', to_encoded_b64, to_encoded_qp, raw_to_needs_mime),
+  score = 1.0,
+  description = 'To header needs encoding',
+  group = 'header'
+}
+
 -- Detects that there is no space in From header (e.g. Some Name<some@host>)
 reconf['R_NO_SPACE_IN_FROM'] = {
   re = 'From=/\\S<[-\\w\\.]+\\@[-\\w\\.]+>/X',
@@ -42,35 +62,34 @@ reconf['R_NO_SPACE_IN_FROM'] = {
   group = 'header'
 }
 
-rspamd_config.MISSING_SUBJECT = {
+reconf['TO_WRAPPED_IN_SPACES'] = {
+  re = [[To=/<\s[-.\w]+\@[-.\w]+\s>/X]],
   score = 2.0,
-  description = 'Subject is missing inside message',
+  description = 'To address is wrapped in spaces inside angle brackets (e.g. display-name < local-part@domain >)',
+  group = 'header'
+}
+
+-- Detects missing Subject header
+reconf['MISSING_SUBJECT'] = {
+  re = '!raw_header_exists(Subject)',
+  score = 2.0,
+  description = 'Subject header is missing',
+  group = 'header'
+}
+
+rspamd_config.EMPTY_SUBJECT = {
+  score = 1.0,
+  description = 'Subject header is empty',
   group = 'header',
   callback = function(task)
     local hdr = task:get_header('Subject')
-
-    if not hdr or #hdr == 0 then
+    if hdr and #hdr == 0 then
       return true
     end
-
     return false
   end
 }
 
--- Detects bad content-transfer-encoding for text parts
--- For text parts (text/plain and text/html mainly)
-local r_ctype_text = 'content_type_is_type(text)'
--- Content transfer encoding is 7bit
-local r_cte_7bit = 'compare_transfer_encoding(7bit)'
--- And body contains 8bit characters
-local r_body_8bit = '/[^\\x01-\\x7f]/Qr'
-reconf['R_BAD_CTE_7BIT'] = {
-  re = string.format('(%s) & (%s) & (%s)', r_ctype_text, r_cte_7bit, r_body_8bit),
-  score = 2.0,
-  description = 'Detects bad content-transfer-encoding for text parts',
-  group = 'header'
-}
-
 -- Detects missing To header
 reconf['MISSING_TO'] = {
   re = '!raw_header_exists(To)',
@@ -107,8 +126,8 @@ reconf['R_RCVD_SPAMBOTS'] = {
 
 -- Charset is missing in message
 reconf['R_MISSING_CHARSET'] = {
-  re = string.format('content_type_is_type(text) & !content_type_has_param(charset) & !%s',
-    r_cte_7bit),
+  re = string.format('!is_empty_body() & content_type_is_type(text) & content_type_is_subtype(plain) & !content_type_has_param(charset) & !%s',
+    'compare_transfer_encoding(7bit)'),
   score = 2.5,
   description = 'Charset is missing in a message',
   group = 'header'
@@ -159,13 +178,9 @@ reconf['TRACKER_ID'] = {
   group = 'header'
 }
 
-
--- From that contains encoded characters while base 64 is not needed as all symbols are 7bit
--- Regexp that checks that From header is encoded with base64 (search in raw headers)
-local from_encoded_b64 = 'From=/\\=\\?\\S+\\?B\\?/iX'
 -- From contains only 7bit characters (parsed headers are used)
 local from_needs_mime = 'From=/[\\x00-\\x08\\x0b\\x0c\\x0e-\\x1f\\x7f-\\xff]/Hr'
--- Final rule
+-- From that contains encoded characters while base 64 is not needed as all symbols are 7bit
 reconf['FROM_EXCESS_BASE64'] = {
   re = string.format('%s & !%s', from_encoded_b64, from_needs_mime),
   score = 1.5,
@@ -174,9 +189,6 @@ reconf['FROM_EXCESS_BASE64'] = {
 }
 
 -- From that contains encoded characters while quoted-printable is not needed as all symbols are 7bit
--- Regexp that checks that From header is encoded with quoted-printable (search in raw headers)
-local from_encoded_qp = 'From=/\\=\\?\\S+\\?Q\\?/iX'
--- Final rule
 reconf['FROM_EXCESS_QP'] = {
   re = string.format('%s & !%s', from_encoded_qp, from_needs_mime),
   score = 1.2,
@@ -184,12 +196,9 @@ reconf['FROM_EXCESS_QP'] = {
   group = 'excessqp'
 }
 
--- To that contains encoded characters while base 64 is not needed as all symbols are 7bit
--- Regexp that checks that To header is encoded with base64 (search in raw headers)
-local to_encoded_b64 = 'To=/\\=\\?\\S+\\?B\\?/iX'
 -- To contains only 7bit characters (parsed headers are used)
 local to_needs_mime = 'To=/[\\x00-\\x08\\x0b\\x0c\\x0e-\\x1f\\x7f-\\xff]/Hr'
--- Final rule
+-- To that contains encoded characters while base 64 is not needed as all symbols are 7bit
 reconf['TO_EXCESS_BASE64'] = {
   re = string.format('%s & !%s', to_encoded_b64, to_needs_mime),
   score = 1.5,
@@ -198,8 +207,6 @@ reconf['TO_EXCESS_BASE64'] = {
 }
 
 -- To that contains encoded characters while quoted-printable is not needed as all symbols are 7bit
--- Regexp that checks that To header is encoded with quoted-printable (search in raw headers)
-local to_encoded_qp = 'To=/\\=\\?\\S+\\?Q\\?/iX'
 -- Final rule
 reconf['TO_EXCESS_QP'] = {
   re = string.format('%s & !%s', to_encoded_qp, to_needs_mime),
@@ -576,6 +583,32 @@ reconf['MAIL_RU_MAILER'] = {
   group = 'header'
 }
 
+-- Detect yandex.ru web-mail
+local xm_yandex_ru_mailer_5_0 = 'X-Mailer=/^Yamail \\[ http:\\/\\/yandex\\.ru \\] 5\\.0$/H'
+local rcvd_web_yandex_ru = 'Received=/^by web\\d{1,2}[a-z]\\.yandex\\.ru with HTTP;/mH'
+reconf['YANDEX_RU_MAILER'] = {
+  re = string.format('(%s) & (%s)', xm_yandex_ru_mailer_5_0, rcvd_web_yandex_ru),
+  score = 0.0,
+  description = 'Sent with yandex.ru web-mail',
+  group = 'header'
+}
+
+-- Detect 1C v8.2 and v8.3 mailers
+reconf['MAILER_1C_8'] = {
+    re = 'X-Mailer=/^1C:Enterprise 8\\.[23]$/H',
+    score = 0.0,
+    description = 'Sent with 1C:Enterprise 8',
+    group = 'header'
+}
+
+-- Detect rogue 'strongmail' MTA with IPv4 and '(-)' in Received line
+reconf['STRONGMAIL'] = {
+    re = [[Received=/^from\s+strongmail\s+\(\[\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\]\) by \S+ \(-\); /mH]],
+    score = 6.0,
+    description = 'Sent via rogue "strongmail" MTA',
+    group = 'header'
+}
+
 -- Two received headers with ip addresses
 local double_ip_spam_1 = 'Received=/from \\[\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\] by \\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3} with/H'
 local double_ip_spam_2 = 'Received=/from\\s+\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\s+by\\s+\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3};/H'
@@ -609,7 +642,7 @@ local ua_pan = 'User-Agent=/^Pan/H'
 local ua_xnews = 'User-Agent=/^Xnews/H'
 local no_inr_yes_ref = string.format('(%s) | (%s) | (%s) | (%s) | (%s) | (%s) | (%s) | (%s) | (%s) | (%s) | (%s)', xm_gnus, xm_msoe5, xm_msoe6, xm_moz4, xm_skyri, xm_wwwmail, ua_gnus, ua_knode, ua_mutt, ua_pan, ua_xnews)
 local subj_re = 'Subject=/^R[eE]:/H'
-local has_ref = 'header_exists(References)'
+local has_ref = '(header_exists(References) | header_exists(In-Reply-To))'
 local missing_ref = string.format('!(%s)', has_ref)
 -- Fake reply (has RE in subject, but has not References header)
 reconf['FAKE_REPLY_C'] = {
@@ -858,14 +891,14 @@ reconf['SUBJECT_HAS_EXCLAIM'] = {
 }
 
 reconf['SUBJECT_ENDS_QUESTION'] = {
-  re = 'Subject=/\\?\\s*$/H',
+  re = 'Subject=/\\?\\s*$/Hu',
   description = 'Subject ends with a question',
   score = 1.0,
   group = 'headers'
 }
 
 reconf['SUBJECT_HAS_QUESTION'] = {
-  re = string.format('%s & !%s', 'Subject=/\\?/H', 'Subject=/\\?\\s*$/H'),
+  re = string.format('%s & !%s', 'Subject=/\\?/H', 'Subject=/\\?\\s*$/Hu'),
   description = 'Subject contains a question',
   score = 0.0,
   group = 'headers'
@@ -899,3 +932,42 @@ reconf['X_PHPOS_FAKE'] = {
   group = 'headers'
 }
 
+reconf['HAS_XOIP'] = {
+  re = "header_exists('X-Originating-IP')",
+  description = "Has X-Originating-IP header",
+  score = 0.0,
+  group = 'headers'
+}
+
+reconf['HAS_LIST_UNSUB'] = {
+  re = string.format('%s', 'header_exists(List-Unsubscribe)'),
+  description = 'Has List-Unsubscribe header',
+  score = -0.01,
+  group = 'headers'
+}
+
+reconf['HAS_GUC_PROXY_URI'] = {
+   re = '/\\.googleusercontent\\.com\\/proxy/{url}i',
+   description = 'Has googleusercontent.com proxy URI',
+   score = 0.01,
+   group = 'experimental'
+}
+
+reconf['HAS_GOOGLE_REDIR'] = {
+  re = '/\\.google\\.com\\/url\\?/{url}i',
+  description = 'Has google.com/url redirection',
+  score = 0.01,
+  group = 'experimental'
+}
+
+reconf['XM_UA_NO_VERSION'] = {
+  re = string.format('(!%s && !%s) && (%s || %s)',
+                     'X-Mailer=/https?:/H',
+                     'User-Agent=/https?:/H',
+                     'X-Mailer=/^[^0-9]+$/H',
+                     'User-Agent=/^[^0-9]+$/H'),
+  description = 'X-Mailer/User-Agent has no version',
+  score = 0.01,
+  group = 'experimental'
+}
+
